#! /usr/bin/rebol
REBOL [
	Title:		"OpenCV Binding"
	Author:		"Franois Jouen"
	Rights:		"Copyright (c) 2012 Franois Jouen. All rights reserved."
	License: {
		Redistribution and use in source and binary forms, with or without modification,
		are permitted provided that the following conditions are met:

		    * Redistributions of source code must retain the above copyright notice,
		      this list of conditions and the following disclaimer.
		    * Redistributions in binary form must reproduce the above copyright notice,
		      this list of conditions and the following disclaimer in the documentation
		      and/or other materials provided with the distribution.

		THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
		ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
		FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
		DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
		SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
		CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
		OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
		OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	}
]

; this lib contains all structures we need for accessing pointers with rebol


; first of all some pointers 
;int-struct: make struct! [int [integer!]] none
;string-struct: make struct! [string [char*]] none
;struct-struct: make struct! [struct [struct! [[save] c [char]]]] none

int*: make struct! [int [integer!]] none
char*: make struct! [buffer [string!]] none
CvArr*: make struct! [ptr [integer!]] none
CvRNG*: make struct! [rng [decimal!]] none
struct*: make struct! [struct [struct! [[save] c [char]]]] none


malloc: func [size [integer!] "size in bytes"][head insert/dup copy {} zero-char size]


;Nenad Rakocevic' routine for memory release
free-mem: func ['word] [set :word make none! recycle]

;there are fabulous functions written by Ladislav Mecir; Thanks a lot Ladislav 
; I have introduced some other news

address?: function [
    {get the address of a string}
    s [any-string!]
] [address] [
    s: make struct! [s [string!]] reduce [s]
    address: make struct! [i [integer!]] none
    change third address third s
    address/i
]

address*: func [
		{get a copy of the nul-terminated string at the given address}
		address [integer!]
	] [
		int*/int: address
		change third char* third int*
		char*/buffer
]

struct-address?: func [
		{get the address of the given struct}
		struct [struct!]
	] [
		address? third struct
]

	
get-mem?: func [
    {get the byte from a memory address}
    address [integer!]
    /nts {a null-terminated string}
    /part {a binary with a specified length}
    length [integer!]
] [
    use [m] [
        address: make struct! [i [integer!]] reduce [address]
        if nts [
            m: make struct! [s [string!]] none
            change third m third address
            return m/s
        ]
        if part [
            m: head insert/dup copy [] [. [char!]] length
            m: make struct! compose/deep [bin [struct! (reduce [m])]] none
            change third m third address
            return third m/bin
        ]
        m: make struct! [c [struct! [chr [char!]]]] none
        change third m third address
        m/c/chr
    ]
]

set-mem: function [
    {set a byte at a specific memory address}
    address [integer!]
    value [char!]
] [m] [
    address: make struct! [i [integer!]] reduce [address]
    m: make struct! [c [struct! [chr [char!]]]] none
    change third m third address
    m/c/chr: value
]

convert: func [
    x
    /to type [word!]
    /local y
] [
    any [to type: type?/word x]
    y: make struct! compose/deep [x [(type)]] reduce [x]
    third y
]

reverse-conversion: func [
    x [binary!]
    type [word!]
    /local y
] [
    y: make struct! compose/deep [x [(type)]] none
    change third y x
    y/x
]

sizeof: func [
		{get the size of a datatype in bytes}
		datatype [word! block!]
	] [
		length? third make struct! compose/deep [value [(datatype)]] none
]


; adapted from Eric Long
bin-to-decimal: func [
    {convert a binary native into a decimal value - also accepts a binary
    string representation in the format returned by REAL/SHOW}
    in      [binary! string!]
    /rev    "binary input in reverse order"
    /local sign exponent fraction
][
    in: copy in
    either binary? in [
        if rev [reverse in]
        sign: either zero? to integer! (first in) / 128 [1][-1]
        exponent: (first in) // 128 * 16 + to integer! (second in) / 16
        fraction: to decimal! (second in) // 16
        in: skip in 2
        loop 6 [
            fraction: fraction * 256 + first in
            in: next in
        ]
    ][
        set [sign exponent fraction] parse in none
        sign: either sign = "0" [1][-1]
        exponent: from-bin exponent
        fraction: from-bin fraction
    ]
    sign * either zero? exponent [
        2 ** -1074 * fraction
    ][
        2 ** (exponent - 1023) * (2 ** -52 * fraction + 1)
    ]
]


; test for structures
size: make struct! [
	x	[integer!] "x value"
	y	[integer!] "y value"
	z   [decimal!] "zvalue"
] [0 0 0]

xy: make struct! [
	x	[integer!] "x value"
	y	[integer!] "y value"
	zz  [decimal!] " cc"
	s	[string!] ;"hello world"
	z	[char!] "z value"
] [10 60 105.0 "hello OpenCv and Rebol world" #"R"] 


cxy: make struct! compose/deep/only[
	x	[integer!] "x value"
	y	[integer!] "y value"
	zz  [decimal!] " cc"
	s	[string!] "hello world"
	ss  [struct! (first size)] "test structure"
	z	[char!] "z value"
	ss2  [struct! (first size)] "test structure"
] [10 60 105.0 "hello double structure"  200 200 500.00 #"R" 10 10 0.0] 



;f. jouen
oldstruct-address*: func [
      {get the data of the struct at the given address. Can use rebol or C types }
		struct [struct!] address [integer!] /endian
	] [
	size: 0
	x: first struct
	data: second struct ; original data in the structure
	foreach v data [size: size + sizeof to-word type? v] ; get size of all types in the structure
	*data: get-mem?/part address size; pointer to data of the structure at the address
	print third struct
	print *data
	ret: copy []
	foreach v data [n: sizeof to-word type? v value: copy/part *data n if endian [reverse value]
	print [type? v n value]
	              switch type?/word v [
	               	char! | char 				[append ret to-char to-integer value]
	               	integer! | int 				[append ret to-integer  value]
	               	short | long				[append ret to-integer  value]
	               	decimal! | float | double	[append ret bin-to-decimal value] ;
	               	string! | char* 			[append ret  address* (to-integer value)] ; string address on 4 bytes
	              ]
	            *data: skip *data n
	] ; get pointed data according to type? 
	return ret; 
]

struct-address*: func [
      {get the data of the struct at the given address. Can use rebol or C types }
		struct [struct!] address [integer!] /endian
	] [
	size: 0
	tmp: first struct
	data: second struct ; original data in the structure
	foreach v data [ size: size + sizeof to-word type? v] ; get size of all types in the structure
	structures: copy []
	foreach s tmp  [
				t: type? s if ( t/word = block!) [
					bloc: copy [] code: to-integer length? s 
					if code = 1 [append bloc code append bloc s append/only structures bloc] ; structure mre
					; structures fille (1 ou n)
					if code > 1 [ stn: second s
						foreach ss stn [
						    t: type? ss if (t/word = block!) [bloc: copy [] append bloc code append bloc ss append/only structures bloc]
						] 
					]
				]
	]
	foreach s structures [print [ s sizeof second s]]
	
	
	*data1: third struct
	
	*data: get-mem?/part address size; pointer to data of the structure at the address
    print *data
    print *data1
	ret: copy []
	count: 0
	foreach v data [
		count: count + 1 
		code: first structures/(count)
		if code = 1 [n: sizeof to-word type? v value: copy/part *data1 n if endian [reverse value
	    print [code type? v n value]
	              switch type?/word v [
	               	char! | char 				[append ret to-char to-integer value]
	               	;integer! | int 			[either code = 1 [append ret to-integer value] [append ret  address* (to-integer value)]]
	               	integer! | int 				[append ret to-integer value]
	               	short | long				[append ret to-integer  value]
	               	decimal! | float | double	[append ret bin-to-decimal value] ;
	               	string! | char* 			[append ret  address* (to-integer value)] ; string address on 4 bytes
	              ]
	            
	            *data1: skip *data1 n
	    ]]
	] ; get pointed data according to type? 
	return ret
	
]





