#! /usr/bin/rebol
REBOL[
	Title:		"OpenCV cxtypes"
	Author:		"Franois Jouen"
	Rights:		"Copyright (c) 2012 Franois Jouen. All rights reserved."
	License: {
		Redistribution and use in source and binary forms, with or without modification,
		are permitted provided that the following conditions are met:

		    * Redistributions of source code must retain the above copyright notice,
		      this list of conditions and the following disclaimer.
		    * Redistributions in binary form must reproduce the above copyright notice,
		      this list of conditions and the following disclaimer in the documentation
		      and/or other materials provided with the distribution.

		THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
		ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
		FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
		DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
		SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
		CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
		OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
		OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	}
]
;we need some pointers
do %cvpointers.r

;from cxtypes.h
;/****************************************************************************************\
;*                             Common macros and inline functions                         *
;\****************************************************************************************/

set 'CV_PI   3.1415926535897932384626433832795
set 'CV_LOG2 0.69314718055994530941723212145818


CV_SWAP: func [a [number!] b [number!] c [number!] /local v1 v2 v3] [
		v1: b
		v2: c
		v3: a
		return [v1 v2 v3]
]
; for MIN and MAX use Rebol functions 

;/* min & max without jumps */
CV_IMIN: func [a [number!] b [number!]  /local val ]
	[ either a < (b - 1) [val: 1] [val: 0]
	return [a or (b or a) and val]
]

CV_IMAX: func [a [number!] b [number!]  /local val ]
	[ either a > (b - 1) [val: 1] [val: 0]
	return [a or (b or a) and val]
]
; some of these function are similar to math rebol functions
;Keep these function in order to remain compatible with OPenCV sources

CV_IABS: func [value /local val][ val: abs value return val]

cvSqrt: func [value  [number!]/local val] [ val: square-root value return val]

cvInvSqrt: func [value  [number!]/local val] [ val: 1.0 / square-root value return val]


; for cvRound cvFloor (Largest integer not greater) and cvCeil (Smallest integer not less) please use rebol function  round/to 

CV_CMP: func [
	a 		[number!]
	b 		[number!] 
	/local val1 val2 
	]
	[ either a > (b) [val1: 1] [val1: 0]
	  either a < (b) [val2: 1] [val2: 0]
	  return [val1 - val2]
]

;#define  CV_SIGN [0 =]


;random 
cvRNG: func [seed] [random/seed seed] 

cvRandInt: func [val [integer!]] [random val]
cvRandReal: func ["return a decimal value beween 0 and 1. Base 16 bit" ] [
	x: random power 2 16
	return x / power 2 16
]

cvIsNaN: func [value [decimal!] /local result] [
		value: 0
		if ( shift value 32) AND #7FFFFFFF = #7FF00000 [value: 1]
        if value <> 0 [value: 1]
	    either value = 2 [return true] [return false]
]


cvIsInf: func [value [decimal!] /local result] [
        value: 0
		if ( shift value 32) AND #7FFFFFFF = #7FF00000 [value: 1]
        if value = 0 [value: 1]
	    either value = 2 [return true] [return false]
]





;/****************************************************************************************\
;*                                  Image type (IplImage)                                 *
;\****************************************************************************************/

 {* The following definitions (until #endif)
 * is an extract from IPL headers.
 * Copyright (c) 1995 Intel Corporation.}
IPL_DEPTH_SIGN: to-integer #80000000

IPL_DEPTH_1U:     1
IPL_DEPTH_8U:     8
IPL_DEPTH_16U:   16
IPL_DEPTH_32F:   32

IPL_DEPTH_8S:  (IPL_DEPTH_SIGN OR 8)
IPL_DEPTH_16S: (IPL_DEPTH_SIGN OR 16)
IPL_DEPTH_32S: (IPL_DEPTH_SIGN OR 32)

IPL_DATA_ORDER_PIXEL:  0
IPL_DATA_ORDER_PLANE:  1

IPL_ORIGIN_TL: 0
IPL_ORIGIN_BL: 1

IPL_ALIGN_4BYTES:   4
IPL_ALIGN_8BYTES:   8
IPL_ALIGN_16BYTES: 16
IPL_ALIGN_32BYTES: 32

IPL_ALIGN_DWORD:   IPL_ALIGN_4BYTES
IPL_ALIGN_QWORD:   IPL_ALIGN_8BYTES

IPL_BORDER_CONSTANT:   0
IPL_BORDER_REPLICATE:  1
IPL_BORDER_REFLECT:    2
IPL_BORDER_WRAP:       3


; basic IPL image structure
_imageId!: make struct! [buffer [string!]] none 
_IplTileInfo!: make struct! [ptr [integer!]] none
_IplImage!: make struct! [buffer [string!]] none

IplROI!: make struct! [
    coi [integer!] ;0 - no COI (all channels are selected), 1 - 0th channel is selected ..
    xOffset [integer!]
    yOffset [integer!]
    width[integer!]
    height[integer!]
] none



IplImage!: make struct! compose/deep/only [
    nSize [integer!]								; sizeof(IplImage)
    ID [integer!]									; version (=0)
    nChannels [integer!]							; Most of OpenCV functions support 1,2,3 or 4 channels
   	alphaChannel [integer!]							; Ignored by OpenCV */
    depth [integer!]								; Pixel depth in bits: 
    colorModel [string!]							;Ignored by OpenCV
    channelSeq [string!]							;ditto *
    dataOrder [integer!]							;0 - interleaved color channels, 1 - separate color channels.
    origin [integer!]								;0 - top-left origin, 1 - bottom-left origin (Windows bitmaps style). 
    align [integer!]								;Alignment of image rows (4 or 8).OpenCV ignores it and uses widthStep instead.
    width [integer!]								;Image width in pixels
    height [integer!]								;Image height in pixels. `
    *roi [struct! [(first IplROI!)]]				;Image ROI. If NULL, the whole image is selected                          	
    *maskROI [struct! [(first _IplImage!)]]			;Must be NULL.
    *imageId [struct! [(first _imageId!)]]			;"           " 
    *tileInfo [integer!]		;"           "
    imageSize [integer!]							; Image data size in bytes
    *imageData [integer!]							; Pointer to aligned image data.         
    widthStep [integer!]							; Size of aligned image row in bytes.    
    BorderMode [integer!]							; Ignored by OpenCV.                     
    BorderConst [integer!]							;  Ditto.                                
    *imageDataOrigin [integer!]						;Pointer to very origin of image data
   
] none 

IplConvKernel!: struct! [
	nCols [integer!]
    nRows [integer!]
    anchorX [integer!]
    anchorY [integer!]
    *values [integer!]
    nShiftR [integer!]
]

IplConvKernelFP!: struct! [
	nCols [integer!]
    nRows [integer!]
    anchorX [integer!]
    anchorY [integer!]
    *values [integer!]
]

IPL_IMAGE_HEADER: 1
IPL_IMAGE_DATA:   2
IPL_IMAGE_ROI:    4

;/* extra border mode */
IPL_BORDER_REFLECT_101:    4
IPL_IMAGE_MAGIC_VAL:  [size? IplImage!]
CV_TYPE_NAME_IMAGE: "opencv-image"
IPL_DEPTH_64F:  64 ; ;/* for storing double-precision floating point data in IplImage's */

CV_IS_IMAGE_HDR:	[not none? IplImage! and IplImage!/nSize = size? IplImage!]
CV_IS_IMAGE:		[ not none? IplImage!/imageData] 

;for storing double-precision floating point data in IplImage's
IPL_DEPTH_64F:  64

;/* get reference to pixel at (col,row), for multi-channel images (col) should be multiplied by number of channels */
CV_IMAGE_ELEM: [IplImage!/imageData + IplImage!/widthStep  * row * col]

{/****************************************************************************************\
*                                  Matrix type (CvMat)                                   *
\****************************************************************************************/}
CV_CN_MAX: 		64
CV_CN_SHIFT: 	3
CV_DEPTH_MAX: 	shift/left 1 CV_CN_SHIFT 1
CV_8U: 			0
CV_8S: 			1
CV_16U: 		2
CV_16S: 		3
CV_32S: 		4
CV_32F: 		5
CV_64F: 		6
CV_USRTYPE1: 	7

CV_MAKETYPE: func [
		depth [integer!]
		cn [integer!]
][
	(depth + cn - 1) shift/left CV_CN_SHIFT 1
]

;#define CV_MAKE_TYPE CV_MAKETYPE
CV_8UC2: CV_MAKETYPE CV_8U 2

_n: make integer!
CV_8UC1: 					CV_MAKETYPE CV_8U 1
CV_8UC2: 					CV_MAKETYPE CV_8U 2
CV_8UC3: 					CV_MAKETYPE CV_8U 3
CV_8UC4: 					CV_MAKETYPE CV_8U 4
CV_8UC_n: 					CV_MAKETYPE CV_8U _n
CV_8SC1: 					CV_MAKETYPE CV_8S 1
CV_8SC2: 					CV_MAKETYPE CV_8S 2
CV_8SC3: 					CV_MAKETYPE CV_8S 3
CV_8SC4: 					CV_MAKETYPE CV_8S 4
CV_8SC_n: 					CV_MAKETYPE CV_8S _n
CV_16UC1: 					CV_MAKETYPE CV_16U 1
CV_16UC2: 					CV_MAKETYPE CV_16U 2
CV_16UC3: 					CV_MAKETYPE CV_16U 3
CV_16UC4: 					CV_MAKETYPE CV_16U 4
CV_16UC_n: 					CV_MAKETYPE CV_16U _n
CV_16SC1: 					CV_MAKETYPE CV_16S 1
CV_16SC2: 					CV_MAKETYPE CV_16S 2
CV_16SC3: 					CV_MAKETYPE CV_16S 3
CV_16SC4: 					CV_MAKETYPE CV_16S 4
CV_16SC_n: 					CV_MAKETYPE CV_16S _n
CV_32SC1: 					CV_MAKETYPE CV_32S 1
CV_32SC2: 					CV_MAKETYPE CV_32S 2
CV_32SC3: 					CV_MAKETYPE CV_32S 3
CV_32SC4: 					CV_MAKETYPE CV_32S 4
CV_32SC_n: 					CV_MAKETYPE CV_32S _n
CV_32FC1: 					CV_MAKETYPE CV_32F 1
CV_32FC2: 					CV_MAKETYPE CV_32F 2
CV_32FC3: 					CV_MAKETYPE CV_32F 3
CV_32FC4: 					CV_MAKETYPE CV_32F 4
CV_32FC_n: 					CV_MAKETYPE CV_32F _n
CV_64FC1: 					CV_MAKETYPE CV_64F 1
CV_64FC2: 					CV_MAKETYPE CV_64F 2
CV_64FC3: 					CV_MAKETYPE CV_64F 3
CV_64FC4: 					CV_MAKETYPE CV_64F 4
CV_64FC_n: 					CV_MAKETYPE CV_64F _n

CV_AUTO_STEP:  				#7FFFFFFF					 
CV_WHOLE_ARR:  				[0 #3FFFFFFF]

CV_MAT_CN_MASK:          	(CV_CN_MAX - 1) shift/left CV_CN_SHIFT 1
CV_MAT_DEPTH_MASK:       	CV_DEPTH_MAX - 1
CV_MAT_TYPE_MASK:        	CV_DEPTH_MAX * CV_CN_MAX - 1
CV_MAT_CONT_FLAG_SHIFT: 	14
CV_MAT_CONT_FLAG:       	1 shift/left CV_MAT_CONT_FLAG_SHIFT 1
CV_IS_MAT_CONT:				[flags and CV_MAT_CONT_FLAG]
CV_IS_CONT_MAT:         	CV_IS_MAT_CONT
CV_MAT_TEMP_FLAG_SHIFT:  	15
CV_MAT_TEMP_FLAG:        	1 shift/left CV_MAT_TEMP_FLAG_SHIFT 1
CV_MAGIC_MASK:       		#FFFF0000
CV_MAT_MAGIC_VAL:    		#42420000
CV_TYPE_NAME_MAT:    		"opencv-matrix"


CvMat!: make struct! [
	type [integer!]
	step [integer!]
	refcount[integer!]
	data [integer!]
	rows [integer!]
	cols [integer!]
] none

CV_MAT_CN: func [
 	flags [integer!]]
 	[ return (flags and CV_MAT_CN_MASK) shift (CV_CN_SHIFT + 1) 1
] 

CV_MAT_DEPTH: func [
 	flags [integer!]]
 	[ return flags and CV_MAT_DEPTH_MASK]
CV_MAT_TYPE: func [
 	flags [integer!]]
 [ return flags and CV_MAT_TYPE_MASK] 

CV_IS_MAT_CONT: func [
 	flags [integer!]]
 	[ return flags and CV_MAT_CONT_FLAG]  
  
CV_IS_TEMP_MAT: func [
 	flags [integer!]]
 	[ return flags and CV_MAT_TEMP_FLAG]

CV_IS_MAT_HDR: func [
 	mat
 	/local v result]
 	[ 	v: 0 
 		if not none? (mat) [v: v + 1 ]
 	 	if mat/type = CV_MAT_MAGIC_VAL [v: v + 1]
 	 	if mat/cols > 0 [v: v + 1]
 	 	if mat/rows > 0 [v: v + 1]
 	 	either v = 4 [result: true] [result: false]
 	return result
]  

CV_IS_MAT: func [
 	mat
 	/local v result]
 	[v: 0 
 	if not none? (mat/data) [v: v + 1 ]
 	if CV_IS_MAT_HDR mat [v: v + 1 ]
 	either v = 2 [result: true] [result: false]
 	return result
 ] 	
CV_IS_MASK_ARR: func [
 	mat]
 	[return  mat/type and CV_MAT_TYPE_MASK and CV_8S 1 = 0]
 	
CV_ARE_TYPES_EQ: func [
 	mat1 mat2 ]
 	[ return (mat1/type OR mat2/type) and CV_MAT_TYPE_MASK = 0
]

CV_ARE_CNS_EQ: func [
 	mat1 mat2]
 	[ return (mat1/type OR mat2/type) and CV_MAT_CN_MASK = 0
]

CV_ARE_DEPTHS_EQ: func [
 	mat1 mat2]
 	[ return (mat1/type OR mat2/type) and CV_MAT_DEPTH_MASK = 0
]

CV_ARE_SIZES_EQ: func [
 	mat1 mat2
 	/local v result]
 	[ v: 0 
 	 if (mat1/rows = mat2/rows) [v: v + 1]
 	 if (mat1/cols = mat2/cols) [v: v + 1]
 	 either v = 2 [result: true] [result: false]
 	return result
]
CV_IS_MAT_CONST: func [
	mat]
 	[return (mat/rows or mat/cols) = 1]

CV_ELEM_SIZE1: func [
	type [integer!]]
	[ return size? size_t shift 28  1 OR 138682897 shift (CV_MAT_DEPTH type * 4 and 15) 1

]

CV_ELEM_SIZE: func [
	type [integer!]]
	[return  CV_MAT_CN type << size? size_t / 4 * 1 * 16384 OR #3A50 shift (CV_MAT_DEPTH type * 2  and 3) 1
]

{inline constructor. No data is allocated internally!!!
(use together with cvCreateData, or use cvCreateMat instead to
get a matrix with allocated data) !!may be  not adapted to rebol}

cvMat: func  [
		rows [integer!]
		cols [integer!]
		type [integer!]
		data [struct! (first int*)]] ; must be a pointer ?
		[
		m: make struct! cvMat!
		assert (m/type and  CV_MAT_DEPTH_MASK) <= CV_64F
		m/type: CV_MAT_MAGIC_VAL OR CV_MAT_CONT_FLAG OR type
		m/cols = cols;
    	m/rows = rows
    	either rows > 1 [m/step: m/cols * CV_ELEM_SIZE type ][m/step: 0]
    	m/data: data;
    	m/refcount: 0;
   		m/hdr_refcount: 0;
   		return m
]

CV_MAT_ELEM_PTR_FAST: func [
	mat 		
	row 		[integer!]
	col 		[integer!]
	pix_size 	[integer!]]
	[return assert row < mat/rows AND col < mat/cols = mat/data + size_t * mat/step * row + pix_size * col	
]

CV_MAT_ELEM_PTR: func [
	mat
	row 		[integer!]
	col 		[integer!]]
	[ return CV_MAT_ELEM_PTR_FAST mat row col CV_ELEM_SIZE mat/type
]

CV_MAT_ELEM: func [
	mat 		
	elemtype	[integer!]
	row 		[integer!]
	col 		[integer!]]
	[ return CV_MAT_ELEM_PTR_FAST mat row col size? elemtype
]

; note arr: array [2 3] : tableau 2 lignes X 3 col
cvmGet: func [
		mat 	
		row 	[integer!]
		col 	[integer!]
		/local type ]
		[
		type: mat/type AND CV_MAT_TYPE_MASK
		return assert (row < mat/rows) AND (col < mat/cols)
		1.0 * mat/data/value + size_t * mat/step * row/col   
		;attention if type == CV_32FC1 : float else double 
]

cvmSet: func [
		row 	[integer!]
		col 	[integer!]
		value 	[decimal!]
		/local type ]
		[
		type: CV_MAT_TYPE mat/type
		assert (row < mat/rows) AND (col < mat/cols)
		1.0 * mat/data/value + size_t * mat/step * row/col: value 
]
cvCvToIplDepth: func [
	type [integer!]
	/local depth val ]
	[depth: CV_MAT_DEPTH type
	val: 0
	either depth = CV_8S [val: IPL_DEPTH_SIGN] [val: 0]
	either depth = CV_16S [val: IPL_DEPTH_SIGN] [val: 0]
	either depth = CV_32S [val: IPL_DEPTH_SIGN] [val: 0]
    return val 
]

;/****************************************************************************************\
;*                       Multi-dimensional dense array (CvMatND)                          *
;\****************************************************************************************/
CV_MATND_MAGIC_VAL:			   	#42430000
CV_TYPE_NAME_MATND:		    	"opencv-nd-matrix"

CvMatND!: make struct! [
	type [integer!]
	dims [integer!]
	refcount[integer!]
	data [integer!]
	dim [struct! [
		size [integer!]
		step [integer!]]
	]
] none

CV_IS_MATND_HDR: func  [
	mat	
	/local v]
	[ v: 0 if not none? (mat) [v: v + 1]
	  if (mat/type AND CV_MAGIC_MASK) = CV_MATND_MAGIC_VAL [v: v + 1]
	  either v = 2 [true] [false]
	  return v
]

CV_IS_MATND: func [
	mat	/local v]
	[ 	v: 0 
		if not none? (mat/data) [v: v + 1]
	  	if CV_IS_MATND_HDR mat [v: v + 1]
	  	either v = 2 [true] [false]
	  	return v
]

;/****************************************************************************************\
;*                      Multi-dimensional sparse array (CvSparseMat)                      *
;\****************************************************************************************/

CV_SPARSE_MAT_MAGIC_VAL:    #42440000
CV_TYPE_NAME_SPARSE_MAT:    "opencv-sparse-matrix"


CvSparseMat!: make struct! compose/deep/only [
	type 		[integer!]
	dims 		[integer!]
	refcount	[integer!]
	heap 		[struct! [value [integer!]]] ; CvSet*
	hashtable 	[integer!]
	hashsize 	[integer!]
	total 		[integer!]
	valoffset 	[integer!]
	idxoffset 	[integer!]
	size 		[integer!]	
] none

CV_IS_SPARSE_MAT_HDR: func [
	mat	/local v]
	[ 	v: 0 
		if not none? (mat/data) [v: v + 1]
		if (mat/type AND CV_MAGIC_MASK) = CV_SPARSE_MAT_MAGIC_VAL [v: v + 1]
	  	either v = 2 [true] [false]
	  	return v
]
CV_IS_SPARSE_MAT: func [
	mat]
	[
    return CV_IS_SPARSE_MAT_HDR mat
]

;/**************** iteration through a sparse array *****************/

CvSparseNode!: make struct! compose/deep/only [
	hashval		[integer!];
    _next		[integer!] ;CvSparseNode*
] none 

CvSparseNode*: make struct! compose/deep [noeud [struct! [(CvSparseNode!)]]] none



CvSparseMatIterator!: make struct! compose/deep/only [
	mat			[struct! [(first CvSparseMat!)]];
    node		[integer!]; CvSparseNode*
    curidx		[integer!]
] none


CV_NODE_VAL: func [
	mat			;[CvSparseMat!]
	node		;[CvSparseNode!]
	]
	[return mat/valoffset + size? (node)
]

CV_NODE_IDX: func [
	mat			;[CvSparseMat!]
	node		;[CvSparseNode!]
	]
	[return mat/idxoffset + size? (node)
]

;/****************************************************************************************\
;*                                         Histogram                                      *
;\****************************************************************************************/

 CvHistType!: 			make integer! 0
 CV_HIST_MAGIC_VAL:     	#42450000
 CV_HIST_UNIFORM_FLAG:  	1 shift/left 10 1
;indicates whether bin ranges are set already or not 
 CV_HIST_RANGES_FLAG:   	1 shift/left 11 1
 CV_HIST_ARRAY:         	0
 CV_HIST_SPARSE:        	1
 CV_HIST_TREE:          	CV_HIST_SPARSE

;should be used as a parameter only, it turns to CV_HIST_UNIFORM_FLAG of hist->type 
 CV_HIST_UNIFORM: 	       1


CvHistogram!: make struct! compose/deep/only[
    type 			[integer!];
    bins 			[struct! [(first CvArr*)]]; ;must be verified
    thresh			[decimal!] ; [CV_MAX_DIM][2]; /* for uniform histograms */
    thresh2			[decimal!] ; /* for non-uniform histograms */
     mat			[struct! [(first CvMatND!)]] ; /* embedded matrix header for array histograms */
] none

CV_IS_HIST: func [
	hist			;[CvHistogram!]
	/local v]
	[ 	v: 0 
		if not none? (hist) [v: v + 1]
		if hist/type  AND CV_MAGIC_MASK = CV_HIST_MAGIC_VAL [v: v + 1]
		if not none? (hist/bins) [v: v + 1]
		return either v = 3 [true] [false]
]

CV_IS_UNIFORM_HIST: func [
	hist			;[CvHistogram!]
	]
	[return (hist/type AND CV_HIST_UNIFORM_FLAG) <> 0
]

CV_IS_SPARSE_HIST: func [
	hist			;[CvHistogram!] 
	/local v]
	[ v: 0
	if not none? (hist/bins) [v: v + 1]
	if (hist/type AND CV_MAGIC_MASK) = CV_SPARSE_MAT_MAGIC_VAL [v: v + 1]
	  	return either v = 2 [true] [false]
]

CV_HIST_HAS_RANGES: func [
	hist			;[CvHistogram!]
	]
	[return hist/type AND CV_HIST_RANGES_FLAG <> 0
]

;/****************************************************************************************\
;*                      Other supplementary data type definitions                         *
;\****************************************************************************************/

;/*************************************** CvRect *****************************************/     


CvRect!: make struct!  [
	x 		[integer!]
	y 		[integer!]
	width 	[integer!]
	height 	[integer!]
] none


cvRectToROI: func[
	rect		;[cvRect!]
	coi			;[integer!]
	/local roi]
	[roi: make struct! _IplROI!
	roi/xOffset: rect/x
    roi/yOffset: rect/y
    roi/width: rect/width
    roi/height: rect/height
    roi/coi: coi
	return roi
]

cvROIToRect: func [
	roi 		;[_IplROI!]
	/local r]
	[r: make struct cvRect!
	r/x: 		roi/xOffset
	r/y: 		roi/yOffset
	r/width: 	roi/width
	r/height: 	roi/height
	return r
]

;/*********************************** CvTermCriteria *************************************/

CV_TERMCRIT_ITER:    		1
TERMCRIT_NUMBER:	  		CV_TERMCRIT_ITER
CV_TERMCRIT_EPS:     		2

CvTermCriteria!: make struct! [
	type		[integer!] ;  may be combination of  CV_TERMCRIT_ITER CV_TERMCRIT_EPS
	max_iter	[integer!]
	epsilon		[decimal!]
] none

cvTermCriteria: func [
	type		[integer!]
	max_iter	[integer!]
	epsilon 	[decimal!]
	/local t]
	[t: make struct! CvTermCriteria!
	t/type: type
	t/max_iter: max_iter
    t/epsilon: epsilon
	return t
]
;/******************************* CvPoint and variants ***********************************/

CvPoint!: make struct! [
	x [integer!]
	y [integer!]
] none


CvPoint2D32f!: make struct! [
	x [decimal!]
	y [decimal!]
] none

CvPoint3D32f!: make struct! [
	x [decimal!]
	y [decimal!]
	z [decimal!]
] none

CvPoint2D64f!: make struct! [
	x [decimal!]
	y [decimal!]
] none

CvPoint3D64f!: make struct! [
	x [decimal!]
	y [decimal!]
	z [decimal!]
] none

;/******************************** CvSize's & CvBox **************************************/

CvSize!: make struct! [
	width 	[integer!]
	height 	[integer!]
] none

ocvSize: func [width height] [
	s: make struct! cvSize! [0 0]
	s/width: width
	s/height: height
	s
]

cvSize: func [size [pair!]] [
	s: make struct! cvSize! [0 0]
	s/width: first size
	s/height: second size
	return reduce second s
]


CvSize2D32f!: make struct! [
	x [decimal!]
	y [decimal!]
] none

cvBox2D!: make struct! compose/deep/only [
	 center 			[struct! [(first CvPoint2D32f!)]] 	;center of the box 
     size 				[struct! [(first CvSize2D32f!)]]		;box width and length
     angle 				[decimal!]			;angle between the horizontal axis and the first side (i.e. length) in degrees
] none

;/* Line iterator state */

CvLineIterator!: make struct! [
	;pointer to the current point 
    uchar* [struct![ptr [integer!]]]
    ;Bresenham algorithm state */
    err [integer!]
    plus_delta [integer!]
    minus_delta [integer!]
    plus_step [integer!]
    minus_step [integer!]
]none

;/************************************* CvSlice ******************************************/

cvSlice!: make struct! [ 
	start_index [integer!]
	end_index [integer!]
] none

;/************************************* CvScalar *****************************************/
; cvColor is not included in openCV; just facilitation for rebol

cvColor!: make struct! [
	b 		[decimal!]
	g 		[decimal!]
	r 		[decimal!]
	alpha 	[decimal!]
] none



; tableau de 4 float ;double val[4];
cvScalar!: make struct! [
	val0 [decimal!]
	val1 [decimal!]
	val2 [decimal!]
	val3 [decimal!]
] none

cvScalar: func [v0 v1 v2 v3] [
	c: make struct! cvScalar! [0 0 0 0]
	c/val0: v0
	c/val1: v1
	c/val2: v2
	c/val3: v3
	return second c

]


cvRealScalar: func [
	v0 [decimal!] ;should be double
	/local c
	][
	c: make struct! cvScalar! [0 0 0 0]
	c/val0: v0
	c/val1: 0.0
	c/val2: 0.0
	c/val3: 0.0
	return c
]

cvScalarAll: func [
	v0 [decimal!] ;should be double
	/local c
	][
	c: make struct! cvScalar! [0 0 0 0]
	c/val0: v0
	c/val1: v0
	c/val2: v0
	c/val3: v0
	return second c
]


;/****************************************************************************************\
;*                                   Dynamic Data structures                              *
;\****************************************************************************************/

;/******************************** Memory storage ****************************************/

cvMemBlock!: make struct! [
	prev 		[integer!] ;cvMemBlock*
	_next 		[integer!] ;cvMemBlock*
] none

cvMemBlock*: make struct! compose/deep [noeud [struct! [(cvMemBlock!)]]] none

CV_STORAGE_MAGIC_VAL:    #42890000



cvMemStorage!: make struct! compose/deep/only [
	signature 		[integer!]
	bottom 			[integer!]		; cvMemBlock* first allocated block */
	top 			[integer!]		; cvMemBlock* current memory block - top of the stack */
	parent 			[integer!]		; cvMemStorage!* borrows new blocks from */
	block_size 		[integer!]		; block size */
    free_space 		[integer!]		; free space in the current block */
] none

cvMemStorage*: make struct! compose/deep [noeud [struct! [(cvMemStorage!)]]] none

CV_IS_STORAGE: func [
	storage			;[cvMemStorage!]
	/local v]
	[ v: 0
	if not none? (storage) [v: v + 1]
	if (storage/signature AND CV_MAGIC_MASK) = CV_STORAGE_MAGIC_VAL [v: v + 1]
	return either v = 2 [true] [false]
]

cvMemStoragePos: make struct! compose/deep/only [
    top [struct! [(first CvMemBlock!)]]
    free_space [integer!]
] none

;/*********************************** Sequence *******************************************/
CvSeqBlock!: make struct! compose/deep/only [
    prev [integer!]									;previous sequence block :  CvSeqBlock*
    next [integer!]									; next sequence block :  CvSeqBlock*
    start_index [integer!]							; index of the first element in the block + sequence->first->start_index */
    count [integer!]	 							; number of elements in the block */
    data [struct! [(first int*)]]					; pointer to the first element of the block */
] none

 CvSeqBlock*: make struct! compose/deep [noeud [struct! [(CvSeqBlock!)]]] none



CV_TREE_NODE_FIELDS*: make struct! compose/deep/only [
	flags 			[integer!]	;micsellaneous flags
	header_size		[integer!]	;size of sequence header
	h_prev			[integer!]	;pointer previous sequence
	h_next			[integer!]	;pointer next sequence
	v_prev			[integer!]	;pointer 2nd previous sequence
	v_next			[integer!]	;pointer 2nd next sequence 
	
] none

CV_TREE_NODE_FIELDS!: func [node_type] [
 return struct! (first CV_TREE_NODE_FIELDS*!)
]

;[struct![ptr [integer!]]]

CV_SEQUENCE_FIELDS: func [][
    CV_TREE_NODE_FIELDS! cvSeq!
	total 			[integer!]						;total number of elements
	elem_size		[integer!]						;size of sequence element in bytes
	block_max		[integer!]						;maximal bound of the last block
	ptr				[integer!]						;current write pointer
	delta_elems		[integer!]						;how many elements allocated when the seq grows
	storage			[struct![ptr [CvMemStorage!]]]	;where the seq is stored
	free_blocks		[struct![ptr [CvSeqBlock!]]]	;free blocks list
	first_block		[struct![ptr [CvSeqBlock!]]]	; pointer to the first sequence block
	return reduce [ CV_TREE_NODE_FIELDS! cvSeq! total elem_size block_max ptr delta_elems storage free_blocks first_block]
] 


CvSeq!: make struct! compose/deep/only [
    r: :CV_SEQUENCE_FIELDS 
] none

CV_TYPE_NAME_SEQ:             "opencv-sequence"
CV_TYPE_NAME_SEQ_TREE:        "opencv-sequence-tree"

;/*************************************** Set ********************************************/
{
  Set.
  Order is not preserved. There can be gaps between sequence elements.
  After the element has been inserted it stays in the same place all the time.
  The MSB(most-significant or sign bit) of the first field (flags) is 0 iff the element exists.
}

elem_type*: make struct! compose/deep/only [(first int*)] none

CV_SET_ELEM_FIELDS!: make struct! compose/deep/only
[
    flags [integer!]				;                         \
    next_free [struct! [(first elem_type*)]];
] none

CvSetElem!: make struct! [
	CvSetElem* [integer!] ; pointeur 
]none

CV_SET_FIELDS: func [] [
    r: :CV_SEQUENCE_FIELDS  
    free_elems	[struct![(first cvSetElem)]]		;  CvSetElem*  \
    active_count [integer!]	;
    return reduce [r free_elems active_count]
]


CvSet!: make struct! compose/deep/only [
	 CvSet_p [integer!]
]none


CV_SET_ELEM_IDX_MASK:   ((1 shift/left 26 1) - 1)
CV_SET_ELEM_FREE_FLAG:   (1 shift/left 31 1)

;/* Checks whether the element pointed by ptr belongs to a set or not */
CV_IS_SET_ELEM: func [ ptr ] [
	 CvSetElem!/flags >= 0
]

;/************************************* Graph ********************************************/

{
  Graph is represented as a set of vertices.
  Vertices contain their adjacency lists (more exactly, pointers to first incoming or
  outcoming edge (or 0 if isolated vertex)). Edges are stored in another set.
  There is a single-linked list of incoming/outcoming edges for each vertex.

  Each edge consists of:
    two pointers to the starting and the ending vertices (vtx[0] and vtx[1],
    respectively). Graph may be oriented or not. In the second case, edges between
    vertex i to vertex j are not distingueshed (during the search operations).

    two pointers to next edges for the starting and the ending vertices.
    next[0] points to the next edge in the vtx[0] adjacency list and
    next[1] points to the next edge in the vtx[1] adjacency list.
}

CV_GRAPH_EDGE_FIELDS!: make struct! [      
    flags			[integer!];        
    weight			[integer!];    
    _next:			[integer!] ; pointer struct CvGraphEdge*  
    vtx				[integer!] ; pointer struct CvGraphVtx* 
]none

CV_GRAPH_VERTEX_FIELDS!: make struct! [    
    flags				[integer!];                
    _first 				[integer!] ; pointer struct CvGraphEdge* ;
]none


CvGraphEdge!: make struct! compose/deep/only
[
    ptr [struct![ (first CV_GRAPH_EDGE_FIELDS!)]]
] none

CvGraphVtx!: make struct! compose/deep/only 
[
	ptr [struct! [(first CV_GRAPH_VERTEX_FIELDS!)]]
] none

CvGraphVtx2D!: make struct! compose/deep/only
[
    ptrg [struct![(first CV_GRAPH_VERTEX_FIELDS!)]]
    ptr [struct! [(first CvPoint2D32f!)]]
] none

;Graph is "derived" from the set (this is set a of vertices) and includes another set (edges)

CV_GRAPH_FIELDS!: make struct! compose/deep/only [
    r: :CV_SET_FIELDS
    edges [integer!]; pointer CvSet* 
] none


CvGraph!: make struct! compose/deep/only [
    ptr [struct! [(first CV_GRAPH_FIELDS!)]]
] none

CV_TYPE_NAME_GRAPH:			 "opencv-graph"

;/*********************************** Chain/Countour *************************************/

CvChain!: make struct! compose/deep/only [
	r: :CV_SEQUENCE_FIELDS
    origin [struct! [(first CvPoint!)]]
] none

CV_CONTOUR_FIELDS!: make struct! compose/deep/only [
    r: :CV_SEQUENCE_FIELDS 
    rect  [struct![(first CvRect!)]]        
    color [integer!];          
    reserved [integer!];
] none

CvContour!: make struct! compose/deep/only
[
    ptr [struct! [(first CV_CONTOUR_FIELDS!)]]
] none

; CvContour CvPoint2DSeq; ????

;/****************************************************************************************\
;*                                    Sequence types                                      *
;\****************************************************************************************/

CV_SEQ_MAGIC_VAL:             #42990000

CV_IS_SEQ: func [seq] [
    return seq != NULL and  (CvSeq!/seq/flags and CV_MAGIC_MASK) = CV_SEQ_MAGIC_VAL
]
CV_SET_MAGIC_VAL:             #42980000

CV_IS_SET: func [aset]
    [ return aset != NULL and (CvSeq!/aset/flags and CV_MAGIC_MASK) = CV_SET_MAGIC_VAL]
    
CV_SEQ_ELTYPE_BITS:           9
CV_SEQ_ELTYPE_MASK:           (1 shift/left CV_SEQ_ELTYPE_BITS - 1 1)
CV_SEQ_ELTYPE_POINT:          CV_32SC2	;  /* (x,y) */
CV_SEQ_ELTYPE_CODE:           CV_8UC1   ;/* freeman code: 0..7 */
CV_SEQ_ELTYPE_GENERIC:        0
CV_SEQ_ELTYPE_PTR:            CV_USRTYPE1
CV_SEQ_ELTYPE_PPOINT:         CV_SEQ_ELTYPE_PTR ;  /* &(x,y) */
CV_SEQ_ELTYPE_INDEX:          CV_32SC1  ;/* #(x,y) */
CV_SEQ_ELTYPE_GRAPH_EDGE:     0  ;/* &next_o, &next_d, &vtx_o, &vtx_d */
CV_SEQ_ELTYPE_GRAPH_VERTEX:   0  ;/* first_edge, &(x,y) */
CV_SEQ_ELTYPE_TRIAN_ATR:      0  ;/* vertex of the binary tree   */
CV_SEQ_ELTYPE_CONNECTED_COMP: 0  ;/* connected component  */
CV_SEQ_ELTYPE_POINT3D:        CV_32FC3  ;/* (x,y,z)  */
CV_SEQ_KIND_BITS:	          3
CV_SEQ_KIND_MASK:	         ((1 shift/left CV_SEQ_KIND_BITS - 1 1 ) shift/left CV_SEQ_ELTYPE_BITS 1)

;/* types of sequences */
CV_SEQ_KIND_GENERIC:	     	(0 shift/left CV_SEQ_ELTYPE_BITS 1)
CV_SEQ_KIND_CURVE:       		(1 shift/left CV_SEQ_ELTYPE_BITS 1)
CV_SEQ_KIND_BIN_TREE:     		(2 shift/left CV_SEQ_ELTYPE_BITS 1)

;/* types of sparse sequences (sets) */
CV_SEQ_KIND_GRAPH:		        (3 shift/left CV_SEQ_ELTYPE_BITS 1)
CV_SEQ_KIND_SUBDIV2D:    		(4 shift/left CV_SEQ_ELTYPE_BITS 1)
CV_SEQ_FLAG_SHIFT:		        (CV_SEQ_KIND_BITS + CV_SEQ_ELTYPE_BITS)

;/* flags for curves */
CV_SEQ_FLAG_CLOSED:     	(1 shift/left CV_SEQ_FLAG_SHIFT 1)
CV_SEQ_FLAG_SIMPLE:     	(2 shift/left CV_SEQ_FLAG_SHIFT 1)
CV_SEQ_FLAG_CONVEX:     	(4 shift/left CV_SEQ_FLAG_SHIFT 1)
CV_SEQ_FLAG_HOLE:       	(8 shift/left CV_SEQ_FLAG_SHIFT 1)

;/* flags for graphs */
CV_GRAPH_FLAG_ORIENTED:		 (1 shift/left CV_SEQ_FLAG_SHIFT 1)
CV_GRAPH:	                 CV_SEQ_KIND_GRAPH
CV_ORIENTED_GRAPH:	        (CV_SEQ_KIND_GRAPH OR CV_GRAPH_FLAG_ORIENTED)

;/* point sets */
 CV_SEQ_POINT_SET:	    	 (CV_SEQ_KIND_GENERIC OR  CV_SEQ_ELTYPE_POINT)
 CV_SEQ_POINT3D_SET:	     (CV_SEQ_KIND_GENERIC OR  CV_SEQ_ELTYPE_POINT3D)
 CV_SEQ_POLYLINE:	         (CV_SEQ_KIND_CURVE   OR  CV_SEQ_ELTYPE_POINT)
 CV_SEQ_POLYGON:	         (CV_SEQ_FLAG_CLOSED  OR  CV_SEQ_POLYLINE )
 CV_SEQ_CONTOUR:	         CV_SEQ_POLYGON
 CV_SEQ_SIMPLE_POLYGON:	     (CV_SEQ_FLAG_SIMPLE  OR  CV_SEQ_POLYGON )
 
 ;/* chain-coded curves */
CV_SEQ_CHAIN:	           (CV_SEQ_KIND_CURVE  OR CV_SEQ_ELTYPE_CODE)
CV_SEQ_CHAIN_CONTOUR:	   (CV_SEQ_FLAG_CLOSED OR CV_SEQ_CHAIN)

;/* binary tree for the contour */
CV_SEQ_POLYGON_TREE:	    (CV_SEQ_KIND_BIN_TREE  OR CV_SEQ_ELTYPE_TRIAN_ATR)

;/* sequence of the connected components */
CV_SEQ_CONNECTED_COMP:	  (CV_SEQ_KIND_GENERIC  OR CV_SEQ_ELTYPE_CONNECTED_COMP)

;/* sequence of the integer numbers */
CV_SEQ_INDEX:	           (CV_SEQ_KIND_GENERIC  OR CV_SEQ_ELTYPE_INDEX)

CV_SEQ_ELTYPE: func [seq] [return seq/flags AND CV_SEQ_ELTYPE_MASK]
CV_SEQ_KIND: func [seq] [return seq/flags AND CV_SEQ_KIND_MASK ]

;/* flag checking */
 CV_IS_SEQ_INDEX: func [seq] [return (CV_SEQ_ELTYPE(seq) = CV_SEQ_ELTYPE_INDEX) and
                                     (CV_SEQ_KIND(seq) = CV_SEQ_KIND_GENERIC)]

CV_IS_SEQ_CURVE: func [seq] [return CV_SEQ_KIND(seq) = CV_SEQ_KIND_CURVE]
CV_IS_SEQ_CLOSED: func [seq] [return (seq/flags AND CV_SEQ_FLAG_CLOSED) != 0]
CV_IS_SEQ_CONVEX: func [seq] [return (seq/flags AND CV_SEQ_FLAG_CONVEX) != 0]
CV_IS_SEQ_HOLE: func [seq] [return (seq/flags AND CV_SEQ_FLAG_HOLE) != 0]
CV_IS_SEQ_SIMPLE: func [seq] [return (seq/flags AND CV_SEQ_FLAG_SIMPLE) != 0 OR CV_IS_SEQ_CONVEX seq]

;/* type checking macros */
CV_IS_SEQ_POINT_SET: func [seq] [return (CV_SEQ_ELTYPE seq = CV_32SC2) OR (CV_SEQ_ELTYPE seq  = CV_32FC2)]
CV_IS_SEQ_POINT_SUBSET: func [seq] [return  (CV_IS_SEQ_INDEX seq  OR CV_SEQ_ELTYPE seq) = CV_SEQ_ELTYPE_PPOINT]
CV_IS_SEQ_POLYLINE: func [seq] [return (CV_SEQ_KIND seq = CV_SEQ_KIND_CURVE) AND CV_IS_SEQ_POINT_SET seq]
CV_IS_SEQ_POLYGON: func [seq] [return CV_IS_SEQ_POLYLINE seq AND CV_IS_SEQ_CLOSED seq]
CV_IS_SEQ_CHAIN: func [seq] [return (CV_SEQ_KIND seq = CV_SEQ_KIND_CURVE) AND seq/elem_size = 1]
CV_IS_SEQ_CONTOUR: func [seq] [return (CV_IS_SEQ_CLOSED seq AND CV_IS_SEQ_POLYLINE seq) OR CV_IS_SEQ_CHAIN seq]
CV_IS_SEQ_CHAIN_CONTOUR: func [seq] [return CV_IS_SEQ_CHAIN seq  AND CV_IS_SEQ_CLOSED seq]
CV_IS_SEQ_POLYGON_TREE: func [seq] [
	return(CV_SEQ_ELTYPE seq =  CV_SEQ_ELTYPE_TRIAN_ATR) AND (CV_SEQ_KIND seq  =  CV_SEQ_KIND_BIN_TREE)
]
CV_IS_GRAPH: func [seq] [return (CV_IS_SET seq AND CV_SEQ_KIND CvSet!/seq) = CV_SEQ_KIND_GRAPH]
CV_IS_GRAPH_ORIENTED: func [seq][return (seq/flags AND CV_GRAPH_FLAG_ORIENTED) != 0]
CV_IS_SUBDIV2D: func [seq][return (CV_IS_SET seq AND CV_SEQ_KIND CvSet!seq) = CV_SEQ_KIND_SUBDIV2D]

;/****************************************************************************************/
;/*                            Sequence writer & reader                                  */
;/****************************************************************************************/
CV_SEQ_WRITER_FIELDS!: make struct! compose/deep/only [                                    
    header_size [integer!];                                      
    seq			[struct! [(first cvSeq!)]]			;CvSeq* the sequence written
    block 		[struct! [(first CvSeqBlock!)]]		;CvSeqBlock* current block
    ptr			[integer!]							;pointer to free space          
    block_min	[integer!]							; pointer to the beginning of block
    block_max	[integer!]							; pointer to the end of block
] none

CvSeqWriter!: make struct! compose/deep/only [ 
   ptr [struct![(first CV_SEQ_WRITER_FIELDS!)]]
] none


CV_SEQ_READER_FIELDS!: make struct! compose/deep/only [                                    
    header_size [integer!];                                      
    seq			[struct! [(first cvSeq!)]]			;CvSeq* sequence, beign read
    block 		[struct! [(first CvSeqBlock!)]]		;CvSeqBlock* current block
    ptr			[integer!]							;pointer to element be read next           
    block_min	[integer!]							;pointer to the beginning of block
    block_max	[integer!]							;pointer to the end of block
    delta_index [integer!]							;= seq/first/start_index
    prev_elem   [integer!]							;pointer to previous element
] none

CvSeqReader!: make struct! compose/deep/only [ 
   ptr [struct![(first CV_SEQ_READER_FIELDS!)]]
] none

;/****************************************************************************************/
;/*                                Operations on sequences                               */
;/****************************************************************************************/

;revoir
CV_SEQ_ELEM: func [seq elem_type index /local val]
[
	val: 0
	if assert [size? seq/first = Size? CvSeqBlock!] [val: val + 1] 
	if assert [size? seq/elem_size = Size? elem_type] [val: val + 1] 
]

CV_WRITE_SEQ_ELEM_VAR: func [elem_ptr writer]     
[                                                    
    if (writer/ptr >= writer/block_max ) [ cvCreateSeqBlock writer] ;make pointer &writer
    ;memcpy((writer).ptr, elem_ptr, (writer).seq->elem_size);
    writer/ptr: writer/seq/elem_size;
]

CV_WRITE_SEQ_ELEM: func [elem writer]             
[                                                     
    assert [writer/seq/elem_size = size? elem]
    if (writer/ptr >= writer/block_max ) [ cvCreateSeqBlock writer] ;make pointer &writer                                              
    assert [writer/ptr <= writer/block_max - size? elem]
    ;memcpy((writer).ptr, &(elem), sizeof(elem));      
    writer/ptr:  writer/ptr + size? elem                    
]

; move reader position forward */
CV_NEXT_SEQ_ELEM: func [elem_size reader]                
[                                                           
    if (reader/ptr + elem_size >= reader/block_max ) [cvChangeSeqBlock  reader 1 ] ;&reader                                                       
]

;/* move reader position backward */
CV_PREV_SEQ_ELEM: func [elem_size reader]              
[                                                            
    if (reader/ptr - elem_size < reader/block_min) [cvChangeSeqBlock  reader 1 ] ;&reader                                                    
]

;/* read element and move read position forward */
CV_READ_SEQ_ELEM: func [elem_size reader]  
[                                                                                  
    assert [reader/seq/elem_size = size? elem]          
    ;memcpy( &(elem), (reader).ptr, sizeof((elem)));            
    CV_NEXT_SEQ_ELEM size? elem reader                  
]
 
;/* read element and move read position backward */
CV_REV_READ_SEQ_ELEM: func [elem_size reader]                     
[                                                                
    assert [reader/seq/elem_size = size? elem]            
    ;memcpy(&(elem), (reader).ptr, sizeof((elem)));               
    CV_PREV_SEQ_ELEM size? elem reader                     
]


CV_READ_CHAIN_POINT: func [_pt reader]                              
[                                                                       
    _pt: reader.pt                                                
    if (reader/ptr )                                                  
    [                                                                   
        CV_READ_SEQ_ELEM reader/code reader                     
        assert [reader/code and 7 = 0 ]                            
        ;(reader).pt.x += (reader).deltas[(int)(reader).code][0]; 
        ;(reader).pt.y += (reader).deltas[(int)(reader).code][1];       
    ]                                                                  
]

;OK
CV_CURRENT_POINT: func [reader /local p] 
[	
	p: make cvPoint!
	p/x: reader/ptr/x
	p/y: reader/ptr/y
	return p
]

CV_PREV_POINT: func [reader /local p] 
[	
	p: make cvPoint!
	p/x: reader/prev_elem/x
	p/y: reader/prev_elem/y
	return p
]

CV_READ_EDGE: func [pt1 pt2 reader]               
[                                                      
    assert [(size? pt1 = size? CvPoint!) AND          
            (size? pt2 = size? CvPoint) AND          
            (reader/seq/elem_size = size? CvPoint!)] 
    pt1: CV_PREV_POINT reader                   
    pt2: CV_CURRENT_POINT reader                
	reader/prev_elem: reader/ptr                 
    CV_NEXT_SEQ_ELEM size? CvPoint! reader      
]

;/************ Graph macros ************/

;/* returns next graph edge for given vertex */
CV_NEXT_GRAPH_EDGE: func [edge vertex][                              
     assert [(edge/vtx[0] = vertex) OR (edge/vtx[1] = vertex)] 
     edge/next [edge/vtx[1] = vertex]
]

;/****************************************************************************************\
;*             Data structures for persistence (a.k.a serialization) functionality        *
;\****************************************************************************************/
;
;/* "black box" file storage */
;typedef struct CvFileStorage CvFileStorage;

;/* storage flags */
CV_STORAGE_READ:          0
CV_STORAGE_WRITE:         1
CV_STORAGE_WRITE_TEXT:    CV_STORAGE_WRITE
CV_STORAGE_WRITE_BINARY:  CV_STORAGE_WRITE
CV_STORAGE_APPEND:        2


;/* list of attributes */
CvAttrList!: make struct! [
	attr		[string!] 						; NULL-terminated array of (attribute_name,attribute_value) pairs
    _next		[integer!]	; CvAttrList* pointer to next chunk of the attributes list
] none

CvAttrList*: make struct! compose/deep [attribute [struct! [(CvAttrList!)]]] none

cvAttrList: func [attr _next ] [
;( const char** attr CV_DEFAULT(NULL), CvAttrList* next CV_DEFAULT(NULL) )
    l: make struct! CvAttrList! none
    l/attr = attr;
    l/_next = _next;

    return l
]

;struct CvTypeInfo;

CV_NODE_NONE:        0
CV_NODE_INT:         1
CV_NODE_INTEGER:     CV_NODE_INT
CV_NODE_REAL:        2
CV_NODE_FLOAT:       CV_NODE_REAL
CV_NODE_STR:         3
CV_NODE_STRING:      CV_NODE_STR
CV_NODE_REF:         4 ;not used
CV_NODE_SEQ:         5
CV_NODE_MAP:         6
CV_NODE_TYPE_MASK:   7
CV_NODE_TYPE: func [flags] [ return flags AND CV_NODE_TYPE_MASK]
 
; /* file node flags */
CV_NODE_FLOW:        8 ;used only for writing structures to YAML format
CV_NODE_USER:        16
CV_NODE_EMPTY:       32
CV_NODE_NAMED:       64
CV_NODE_IS_INT: func [flags]  [return CV_NODE_TYPE flags = CV_NODE_INT]
CV_NODE_IS_REAL: func [flags]  [return CV_NODE_TYPE flags = CV_NODE_REAL]
CV_NODE_IS_STRING: func [flags] [return CV_NODE_TYPE = CV_NODE_STRING]
CV_NODE_IS_SEQ: func [flags] [return CV_NODE_TYPE = CV_NODE_SEQ]
CV_NODE_IS_MAP: func [flags] [return CV_NODE_TYPE flags = CV_NODE_MAP]
CV_NODE_IS_COLLECTION: func [flags] [return CV_NODE_TYPE flags >= CV_NODE_SEQ]
CV_NODE_IS_FLOW: func [flags] [return (flags AND CV_NODE_FLOW) != 0]
CV_NODE_IS_EMPTY: func [flags] [return(flags AND CV_NODE_EMPTY) != 0]
CV_NODE_IS_USER: func [flags] [return (flags AND CV_NODE_USER) != 0]
CV_NODE_HAS_NAME: func [flags] [return (flags AND CV_NODE_NAMED) != 0]
CV_NODE_SEQ_SIMPLE: 256
CV_NODE_SEQ_IS_SIMPLE: func [seq] [return (seq/flags AND CV_NODE_SEQ_SIMPLE) != 0]

CvString!: make struct! [
	len 	[integer!]
    ptr		[string!]
] none

{all the keys (names) of elements in the readed file storage
are stored in the hash to speed up the lookup operations }


CvStringHashNode!: make struct! [
    hashval			[integer!];
    str				[string!]		; should be CvString!
    _next			[integer!]		;pointer to  CvStringHashNode* ;
] none

CvStringHashNode*: make struct! compose/deep [attribute [struct! [(CvStringHashNode!)]]] none

;typedef struct CvGenericHash CvFileNodeHash;

CvFileNode: make struct! [
    tag			[integer!];
    info 		[integer!]; CvTypeInfo* info; /* type information (only for user-defined object, for others it is 0) */
    data 		[integer!] ; pointer to union
    {   double f; /* scalar floating-point number 
        int i;    /* scalar integer number
        CvString str; /* text string
        CvSeq* seq; /* sequence (ordered collection of file nodes)
        CvFileNodeHash* map; /* map (collection of named file nodes)
    }
] none

CvTypeInfo*: 
CvTypeInfo!: make struct!
[
    flags				[integer!];
    header_size			[integer!];
    _prev				[integer!]	; pointer CvTypeInfo;
    _next				[integer!]	; pointer CvTypeInfo;
    type_name			[string!];
    is_instance			[integer!]	;pointer to function CvIsInstanceFunc
    release				[integer!]	;pointer to function CvReleaseFunc ;
    _read				[integer!]  ;CvReadFunc ;
    _write				[integer!]  ;CvWriteFunc write;
    clone				[integer!]  ;CvCloneFunc ;
] none

CvTypeInfo*: make struct! compose/deep [info [struct! [(CvTypeInfo!)]]] none 

;/**** System data types ******/

CvPluginFuncInfo!: struct! [
    func_addr			[integer!]; pointer
    default_func_addr	[integer!]; pointer
    func_names			[string!];
    search_modules		[integer!];
    loaded_from			[integer!];
] none

CvModuleInfo!:
[
    _next			[integer!]		;pointer CvModuleInfo;
    name			[string!]
    version			[string!]
    func_tab		[integer!]		; pointer CvPluginFuncInfo! ;
]

CvModuleInfo*: make struct! compose/deep [info [struct! [(CvModuleInfo!)]]] none 
